function [ x_out,objective,iter_stop] = ADMM_Lasso_DOA( y,A,rho,lambda,e_abs,e_rel,maxIter)
%ADMM_LASSO for DOA estimation
% ================================================================================================
%   [ x_out,objective,e_pri,e_dual,iter_stop] = ADMM_Lasso( y,A,rho,lambda,e_abs,e_rel,maxIter)
% ================================================================================================
% Solves ||y-Ax||_2^2+lambda||x||_1 using ADMM (1)
% Input:
%       y       - The data vector
%       A       - The dictionary
%       rho     - The augmented Lagrangian parameter
%       lambda  - The scalar regularization parameter
%       e_abs   - The absolute tolerance (Often set to 10^-3 or 10^-4
%       e_rel   - The relative tolerance (Often set to 10^-3 or 10^-4
%       maxIter - The number of maximum iterations
% Output:
%       x_out     - The estiamted x from (1)
%       objective - The resulting objective function (1) at each iteration
%       iter_stop - The last iteration


[M, G] = size(A);
L=size(y,2);
x = zeros(G,L);
z = zeros(G,1);
u = zeros(G,L);

qAi=2/rho.*eye(G)-2/rho*A'*inv(rho/2.*eye(M)+A*A')*A;

counter=1;

objective(counter)=1/2*trace((y-A*x)'*(y-A*x))+lambda*norm(z,1);

X_mean=zeros(G,1);

while maxIter>counter
    X_mean_old=X_mean;
    counter = counter + 1;
    z_old=z;
    u_old = u;

    
    Z=repmat(z_old,1,L);
    x=qAi*(A'*y+rho/2*(Z-u_old));
    
    z_temp=mean(x,2)+mean(u_old,2);
    z = SoftThresh(lambda/(L*rho),z_temp);
    
    Z=repmat(z,1,L);
    u=u_old+x-Z;
    
    X_mean=repmat(mean(x,2),1,L);
    X_mean_old=X_mean;
    %     r(counter)=sum(sum((x-X_mean).*conj(x-X_mean)))/G/L;
    %     s(counter)=trace((X_mean_old-X_mean)'*(X_mean_old-X_mean))/G/L;
%     r(counter)=sum(sum((x-Z).*conj(x-Z)))/G/L;         %primal residual
%     s(counter)=sum(sum(lambda.*norm((z_old-z),1)))/G;          % dual residual

    r(counter)=sqrt(trace((x-Z)'*(x-Z))/G);
    s(counter)=sqrt(trace((z-z_old)'*(z-z_old))/G);
        z_old=z;
    if  r(counter)<1e-4&& s(counter)<1e-4
%         figure
%         stem(z)
        break;
    end
    objective(counter)=1/2*trace((y-A*x)'*(y-A*x))+lambda*norm(z,1);
%     figure
%     stem(z)
end
x_out = z;
%     figure
%     stem(z)
iter_stop = counter-1;
end

function [ S ] = SoftThresh( kappa, alpha )
if alpha==0
    S=zeros(size(alpha));
else
    S = max(1-kappa./abs(alpha),0).*alpha;
end
end